#include "StdAfx.h"
#include "Resource.h"
#include "dxs_MMOGNetWork.h"
#include "dxs_MMOGDlg.h"

// -----------------------------------------------------------------------------
UINT WorkerThread1(LPVOID pParam)
// -----------------------------------------------------------------------------
{
	OutputDebugString("WorkerThread1 started. OK\r\n");

	Cdxs_MMOGNetwork* pNetWork = (Cdxs_MMOGNetwork*) pParam;
	pNetWork->m_bWorkerThread1HasEnded = false;

	while (!pNetWork->m_bShutDownWorkerThread1)
	{
		pNetWork->ProcessNetwork();
		Sleep(10);
	}

	OutputDebugString("WorkerThread1 ended. OK\r\n");
	pNetWork->m_bWorkerThread1HasEnded = true;
	
	return 0;
}

// -----------------------------------------------------------------------
Cdxs_MMOGNetwork::Cdxs_MMOGNetwork(void)
// -----------------------------------------------------------------------
{
	m_ThisClient = NULL;
	m_ThisPeer   = NULL;
	m_bShutDownWorkerThread1 = true;
	m_bWorkerThread1HasEnded = true;
}

// -----------------------------------------------------------------------
Cdxs_MMOGNetwork::~Cdxs_MMOGNetwork(void)
// -----------------------------------------------------------------------
{
	m_bShutDownWorkerThread1 = true;
}

// -----------------------------------------------------------------------
bool Cdxs_MMOGNetwork::InitNetwork()
// -----------------------------------------------------------------------
{
	// Initialize the network
	int iError = enet_initialize();

	if (iError)
	{
		OutputDebugString("Cdxs_MMOGNetwork::InitNetwork: Could not initialize enet. ERROR\r\n");
		return(false);
	}
	
	OutputDebugString("Cdxs_MMOGNetwork::InitNetwork: Enet initialized. OK\r\n");
	return(true);
}

// -----------------------------------------------------------------------
bool Cdxs_MMOGNetwork::CreateLocalClient()
// -----------------------------------------------------------------------
{
	// Create the client
	m_ThisClient = enet_host_create (NULL, 1, 0, 0);

	if (m_ThisClient == NULL)
	{
		OutputDebugString("Cdxs_MMOGNetwork::CreateLocalClient: Could not create network client. ERROR\r\n");
		return(false);
	}
    
	OutputDebugString("Cdxs_MMOGNetwork::CreateLocalClient: Network client created. OK\r\n");
	return(true);
}

// -----------------------------------------------------------------------
bool Cdxs_MMOGNetwork::ConnectToHost()
// -----------------------------------------------------------------------
{
	// Connect to the server
	ENetAddress ServerAddress;
    ENetEvent event;
   
    enet_address_set_host (&ServerAddress, gl_Data.m_sHostIP);
	ServerAddress.port = gl_Data.m_iHostPort;

    // Initiate the connection, allocating the two channels 0 and 1.
    m_ThisPeer = enet_host_connect (m_ThisClient, &ServerAddress, 2);    
    
    if (m_ThisPeer == NULL)
    {
		OutputDebugString("Cdxs_MMOGNetwork::ConnectToHost: Could not connect to server(1). ERROR\r\n");
		return(false);
    }
    
    // Wait up to 2 seconds for the connection attempt to succeed. 
    if ((enet_host_service (m_ThisClient, &event, 2000) > 0) && (event.type == ENET_EVENT_TYPE_CONNECT))
    {
        OutputDebugString("Cdxs_MMOGNetwork::ConnectToHost: Connection to server established. OK\r\n");
		
		m_bShutDownWorkerThread1 = false;
		AfxBeginThread(WorkerThread1,this); // Begin the network listener thread

		return(true);
    }
    else
    {
        enet_peer_reset (m_ThisPeer);
		OutputDebugString("Cdxs_MMOGNetwork::ConnectToHost: Could not connect to server(2). ERROR\r\n");
		return(false);
    }
}

// -----------------------------------------------------------------------
void Cdxs_MMOGNetwork::ExitNetwork()
// -----------------------------------------------------------------------
{
	m_bShutDownWorkerThread1 = true;
	do {} while (!m_bWorkerThread1HasEnded);
	
	// Disconnect the peer
	if (m_ThisPeer)
	{
		ENetEvent event;
        enet_peer_disconnect (m_ThisPeer, 0);

		while (m_ThisPeer && enet_host_service (m_ThisClient, &event, 2000) > 0)
		{
			switch (event.type)
			{
				case ENET_EVENT_TYPE_RECEIVE: enet_packet_destroy (event.packet); break;
				case ENET_EVENT_TYPE_DISCONNECT: OutputDebugString("Cdxs_MMOGNetwork::ExitNetwork: Network client disconnected. OK\r\n"); m_ThisPeer = NULL; break;
			}
        }

		// Timeout. Force disconnect.
		if (m_ThisPeer)
		{
			enet_peer_reset (m_ThisPeer);
			m_ThisPeer = NULL;
			OutputDebugString("Cdxs_MMOGNetwork::ExitNetwork: Network client forced to disconnect. WARNING\r\n");
		}
    }	
	
	// Destroy the network client
	if (m_ThisClient)
	{
		enet_host_destroy(m_ThisClient);
		m_ThisClient = NULL;
		OutputDebugString("Cdxs_MMOGNetwork::ExitNetwork: Network client destroyed. OK\r\n");
	}
	
	// Shut down network service
	enet_deinitialize();
	OutputDebugString("Cdxs_MMOGNetwork::ExitNetwork: Enet deinitialized. OK\r\n");
}

// -----------------------------------------------------------------------
void Cdxs_MMOGNetwork::ProcessNetwork()
// -----------------------------------------------------------------------
{
	if (!m_ThisClient) return;
	
	Cdxs_MMOGDlg* pDlg = (Cdxs_MMOGDlg*) ::AfxGetApp()->m_pMainWnd;

	ENetEvent event;
	if (enet_host_service (m_ThisClient, &event, 0)>0)
	{
		if (event.type==ENET_EVENT_TYPE_DISCONNECT)
		{
			OutputDebugString("Cdxs_MMOGNetwork::ProcessNetwork: disconnect received. ERROR\r\n");
			m_bShutDownWorkerThread1 = true;
			pDlg->disableNetwork();
		}
		else if(event.type==ENET_EVENT_TYPE_RECEIVE)
		{
			pDlg->processNetworkMessage((BYTE*) event.packet->data);			
			enet_packet_destroy (event.packet);
		}
		else enet_packet_destroy (event.packet);
	}
}

// -----------------------------------------------------------------------------
void Cdxs_MMOGNetwork::SendInstanceID(char* sID)
// -----------------------------------------------------------------------------
{
	char dummy[110];
	sprintf(dummy,"ID%s",sID);

	ENetPacket *packet = enet_packet_create(dummy,strlen(dummy)+1,ENET_PACKET_FLAG_RELIABLE);
	enet_peer_send (m_ThisPeer, 0, packet);
}

