
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <sys/time.h>
#include "enet.h"

struct clientData
{
    char instanceID[100];

};

struct network_message
{
    short type;
    float vPos[3];
};

ENetAddress gl_Address;
ENetHost*   gl_Server;
int         gl_Clients;
bool        gl_ExitServer;


// --------------------------------------------------------------
void catchCtrlC(int sig)
// --------------------------------------------------------------
{
   gl_ExitServer = true;
   (void) signal(SIGINT, SIG_DFL);
}


// --------------------------------------------------------------
void exitServer(void)
// --------------------------------------------------------------
{
   printf("Exiting: Cleaning up\n\n");
   if (gl_Server) enet_host_destroy(gl_Server);
   enet_deinitialize();
}


// --------------------------------------------------------------
bool initServer(void)
// --------------------------------------------------------------
{
   if (enet_initialize () != 0)
   {
      printf("Could not initialize network. ERROR\n");
      return(false);
   }
   else
   {
      printf("Network initialized. OK\n");
   }

   // Catch ctrl-c by user
   (void) signal(SIGINT, catchCtrlC);
   			
   gl_Address.host = ENET_HOST_ANY;
   gl_Address.port = 9621;

   gl_Server = enet_host_create (&gl_Address, 32, 0, 0);

   if (gl_Server == NULL)
   {
      printf ("Could not create server. ERROR\n");
      return(false);
   }
   else
   {
     printf("Server created. OK\n");
   }

   return(true);
}

// --------------------------------------------------------------
void createClientData(ENetEvent* pEvent)
// --------------------------------------------------------------
{
   clientData* pData = new clientData;
   memset(pData,0,sizeof(clientData));
   strncpy(pData->instanceID,"((empty))",strlen("((empty))"));
   pEvent->peer->data = (void*) pData;
}

// --------------------------------------------------------------
void removeClientData(ENetEvent* pEvent)
// --------------------------------------------------------------
{
   clientData* pData = (clientData*) pEvent->peer->data;
   delete(pData);
   pEvent->peer->data = NULL;
}

// --------------------------------------------------------------
char* getClientDataInstanceID(ENetEvent* pEvent)
// --------------------------------------------------------------
{
   clientData* pData = (clientData*) pEvent->peer->data;
   return(pData->instanceID);
}

// --------------------------------------------------------------
void setClientDataInstanceID(ENetEvent* pEvent)
// --------------------------------------------------------------
{
   clientData* pData = (clientData*) pEvent->peer->data;
   memset(pData->instanceID,0,100);
   strncpy(pData->instanceID,(const char*) &(pEvent->packet->data[2]),pEvent->packet->dataLength-2);
}

// --------------------------------------------------------------
void sendRookPosition(void)
// --------------------------------------------------------------
{
   static unsigned long last = 0;
   static float fXPos = 0;

   timeval now;
   gettimeofday(&now, NULL);

   if ((now.tv_usec - last)>500)
   {
	last = now.tv_usec;
	fXPos += 0.5;
	if (fXPos > 3) fXPos = -3;

	network_message msg;
	msg.type = 1;
	msg.vPos[0] = fXPos;
	msg.vPos[1] = 0;
	msg.vPos[2] = 0;
	ENetPacket* packet = enet_packet_create (&msg, sizeof(network_message),ENET_PACKET_FLAG_RELIABLE);
	enet_host_broadcast(gl_Server, 0, packet);
   }
}

// --------------------------------------------------------------
int main (int argc, char ** argv)
// --------------------------------------------------------------
{
   printf("\nDXStudio Test Server\n");
   printf("====================\n");
   
   if (gl_ExitServer = !initServer());
      
   gl_Clients = 0;   
   ENetEvent event;
       
   while(!gl_ExitServer)
   {
      if (gl_Clients == 0) usleep(100); // Sleep a bit if no client there

      int iRet = enet_host_service(gl_Server, &event, 100);

      if (iRet > 0)
      {
         printf("Event received!\n");
	 
	 switch (event.type)
         {
	   case ENET_EVENT_TYPE_CONNECT:
	   		gl_Clients++;
			printf ("A new client connected from %x:%u. Clients: %i of %i\n", 
									event.peer->address.host, 
									event.peer->address.port, 
									gl_Clients,
									gl_Server->peerCount);
			createClientData(&event);
			break;

	   case ENET_EVENT_TYPE_DISCONNECT:
                        gl_Clients--;
			printf ("Client %s disconnected. Clients: %i of %i\n", getClientDataInstanceID(&event), 
									       gl_Clients, 
									       gl_Server->peerCount);
                        removeClientData(&event);
			break;

	   case ENET_EVENT_TYPE_RECEIVE:
			printf ("A packet of length %u containing %s was received from %s on channel %u.\n",
			event.packet->dataLength, event.packet->data, getClientDataInstanceID(&event), event.channelID);
	                
			if ((event.packet->dataLength>2)&&(event.packet->data[0]=='I')&&(event.packet->data[1]=='D'))
			{
			  setClientDataInstanceID(&event);
			  printf("Client id stored.\n");
			}
			
			enet_packet_destroy (event.packet);
			break;
         }
      }
      else if (iRet<0) printf("enet_host_service returned ERROR\n");

      // If clients there, send position for the chess rook
      if (gl_Clients) sendRookPosition();
   }

   exitServer();
}
			    
